/*#########################################################################
#   Amarok script for interfacing with SoundCloud                         #
#                                                                         #
#   Copyright                                                             #
#   (C) 2014 Be <be.0@gmx.com>                                            #
#                                                                         #
#   This program is free software; you can redistribute it and/or modify  #
#   it under the terms of the GNU General Public License as published by  #
#   the Free Software Foundation; either version 3 of the License, or     #
#   (at your option) any later version.                                   #
#                                                                         #
#   Note that if you modify this software and continue to use the         #
#   SoundCloud API in your modified version, the SoundCloud API Terms of  #
#   Use (https://developers.soundcloud.com/docs/api/terms-of-use ) state, #
#   in the section General Prohibitions, clause xiv:                      #
#      You must not use or attempt to use another person's client ID      #
#      and/or client secret, unless you are working on the same app.      #
#   You are encouraged to communicate your modifications to the author    #
#   so they can be incorporated into a future version of this program.    #
#   If you wish to publish a modified version, to comply with the         #
#   SoundCloud API Terms of Use, register your new program with           #
#   SoundCloud through their website and replace the value of the global  #
#   variable client_id, defined at the first line of code, with your      #
#   client ID.                                                            #
#                                                                         #
#   This program is distributed in the hope that it will be useful,       #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the          #
#   GNU General Public License for more details.                          #
#                                                                         #
#   You should have received a copy of the GNU General Public License     #
#   along with this program; if not, write to the                         #
#   Free Software Foundation, Inc.,                                       #
#   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         #
#########################################################################*/

var client_id = '182a6730c85291f153ce073e55f0ad9e'

Importer.loadQtBinding("qt.core");
Importer.loadQtBinding("qt.gui");
Importer.loadQtBinding("qt.uitools");

function SoundCloudService() {
	ScriptableServiceScript.call( this, "SoundCloud Integration", 3, "Stream from SoundCloud", "Stream from SoundCloud", true );
}

function onConfigure() {
	// TODO: Username and password dialog
	Amarok.alert( "This script does not require any configuration." );
}

function htmlify (string) {
	if (string == null) {
		return '';
	}
	string = string.replace(/(?:\r\n|\r|\n)/g,'<br/>');
	// URL regex from Android's WEB_URL constant
	urlRegex = /((?:(http|https|Http|Https|rtsp|Rtsp):\/\/(?:(?:[a-zA-Z0-9\$\-\_\.\+\!\*\'\(\)\,\;\?\&\=]|(?:\%[a-fA-F0-9]{2})){1,64}(?:\:(?:[a-zA-Z0-9\$\-\_\.\+\!\*\'\(\)\,\;\?\&\=]|(?:\%[a-fA-F0-9]{2})){1,25})?\@)?)?((?:(?:[a-zA-Z0-9][a-zA-Z0-9\-]{0,64}\.)+(?:(?:aero|arpa|asia|a[cdefgilmnoqrstuwxz])|(?:biz|b[abdefghijmnorstvwyz])|(?:cat|com|coop|c[acdfghiklmnoruvxyz])|d[ejkmoz]|(?:edu|e[cegrstu])|f[ijkmor]|(?:gov|g[abdefghilmnpqrstuwy])|h[kmnrtu]|(?:info|int|i[delmnoqrst])|(?:jobs|j[emop])|k[eghimnrwyz]|l[abcikrstuvy]|(?:mil|mobi|museum|m[acdghklmnopqrstuvwxyz])|(?:name|net|n[acefgilopruz])|(?:org|om)|(?:pro|p[aefghklmnrstwy])|qa|r[eouw]|s[abcdeghijklmnortuvyz]|(?:tel|travel|t[cdfghjklmnoprtvwz])|u[agkmsyz]|v[aceginu]|w[fs]|y[etu]|z[amw]))|(?:(?:25[0-5]|2[0-4][0-9]|[0-1][0-9]{2}|[1-9][0-9]|[1-9])\.(?:25[0-5]|2[0-4][0-9]|[0-1][0-9]{2}|[1-9][0-9]|[1-9]|0)\.(?:25[0-5]|2[0-4][0-9]|[0-1][0-9]{2}|[1-9][0-9]|[1-9]|0)\.(?:25[0-5]|2[0-4][0-9]|[0-1][0-9]{2}|[1-9][0-9]|[0-9])))(?:\:\d{1,5})?)(\/(?:(?:[a-zA-Z0-9\;\/\?\:\@\&\=\#\~\-\.\+\!\*\'\(\)\,\_])|(?:\%[a-fA-F0-9]{2}))*)?(?:\b|$)/gi;
	string = string.replace(urlRegex, function(url) {
		if ( ( url.indexOf(".jpg") > 0 ) || ( url.indexOf(".png") > 0 ) || ( url.indexOf(".gif") > 0 ) ) {
			return '<img src="' + url + '">';
		} else {
			return '<a href="' + url + '">' + url + '</a>';
		}
	})
	return string;
}

function artistList(reply) {
	query = JSON.parse(reply);
	
	if (! ('errors' in query)) {
		item = Amarok.StreamItem;
		item.level = 2;
		for (i in query) {
			item.itemName = query[i].username;
			item.artist = query[i].username;
			if (query[i].track_count > 0) {
				Amarok.debug(item.callbackData);
				item.callbackData = JSON.stringify(query[i]);
				item.infoHtml = htmlify(query[i].description);
			} else {
				item.callbackData = '{"errors": "no tracks"}';
			}
			script.insertItem(item);
			Amarok.debug(item.itemName + ' inserted');
		}
	}
	script.donePopulating();
	Amarok.debug('Artist list populated');
}

var playlistTracks = [];
var artist='';

function playlists(reply) {
	plists = JSON.parse(reply);
	playlistTracks = [];
	if (! (plists.length == 0)) {
		item = Amarok.StreamItem;
		item.level = 1;
		for (i in plists) {
			item.callbackData = JSON.stringify(plists[i].tracks);
			item.year = plists[i].release_year;
			item.itemName = plists[i].title;
			item.artist = plists[i].user.username;
			item.infoHtml = htmlify(plists[i].description);
			item.coverUrl = plists[i].artwork_url;
			script.insertItem(item);
			for (j in plists[i].tracks) {
				playlistTracks.push(plists[i].tracks[j]);
			}
		}
	}
	new Downloader(new QUrl('https://api.soundcloud.com/users/' + artist + '/tracks?client_id=' + client_id ), nonPlaylistTracks);
}

function nonPlaylistTracks (reply) {
	tracklist = JSON.parse(reply);
	cb = JSON.stringify(tracklist.filter(notInPlaylist));
	if (cb != '[]') {
		item = Amarok.StreamItem;
		item.level = 1;
		item.callbackData = cb;
		item.itemName = 'Other tracks';
		item.artist = tracklist[0].user.username;
		item.coverUrl = tracklist[0].user.avatar_url;
		script.insertItem(item);
	}
	script.donePopulating();
}

function notInPlaylist (value, index, array) {
	for (i in playlistTracks) {
		if (value.title == playlistTracks[i].title) {
			return false;
		}
	}
	return true;
}

function tracks(callback) {
	tracklist = JSON.parse(callback);
	item = Amarok.StreamItem;
	item.level = 0;
	item.callbackData = null;
	if (! ('errors' in tracklist)) {
		for (i in tracklist) {
			item.year = tracklist[i].release_year;
			item.itemName = tracklist[i].title;
			item.artist = tracklist[i].user.username;
			item.playableUrl = tracklist[i].stream_url + '?client_id=' + client_id;
			item.infoHtml = trackInfo(tracklist[i]);
			item.coverUrl = tracklist[i].artwork_url;
			script.insertItem(item);
		}
// 	} else if (tracklist.errors == 'no tracks') {
// 		item.itemName = 'No tracks';
// 		item.playableUrl = 'dummy item';
// 		item.artist = artist;
// 		script.insertItem(item);
// 	} else if (tracklist.errors == 'prompt') {
// 		item.itemName = 'Search';
// // 		item.playableUrl = 'http://foo.com/blah.mp3';
// 		script.insertItem(item);
	}
	script.donePopulating();
}

function trackInfo (info) {
	var br = '<br/>';
	html = '' +
	'<div style="text-align:center;">' +
		'<a href="' + info.user.permalink_url + '">' +
			'<img src="' + info.user.avatar_url + '" style="position:absolute;top:0;right:0;"/>' +
			'<h1>' + info.user.username + '</h1>' +
		'</a>' +
		'<h2><a href="' + info.permalink_url + '">' + info.title + '</a></h2>' +
		'Published: ' + info.created_at + br +
		'Plays: ' + info.playback_count + ' | ' +
		'Favorites: ' + info.favoritings_count + ' | ' +
		'Tags: ' + info.tag_list +
	'</div>';
	if (info.description != null) {
		html = html + br + '<div>' + htmlify(info.description) + '</div>';
	}
	return html;
}

function onPopulating( level, callback, filter ) {
	Amarok.debug("Populating level " + level);
	switch (level) {
		case 2:
			Amarok.debug('filter: ' + filter);
			// Amarok adds '%20' to the end of the filter
			if (filter == '') {
				item = Amarok.StreamItem;
				item.level = 2;
				item.itemName = 'Search for an artist...';
				item.infoHtml = 'Search for an artist...';
				item.callbackData = '{"errors": "prompt"}';
				script.insertItem(item);
				script.donePopulating();
			} else {
				new Downloader(new QUrl('https://api.soundcloud.com/users?q=' + filter.slice(0,-3) + '&client_id=' + client_id ), artistList);
			}
			break;
		case 1:
			Amarok.debug(callback);
			artistData = JSON.parse(callback);
			if (! ('errors' in artistData)) {
				artist = artistData.id;
				new Downloader(new QUrl('https://api.soundcloud.com/users/' + artistData.id + '/playlists?client_id=' + client_id ), playlists);
			} else if (artistData.errors == 'no tracks') {
				item = Amarok.StreamItem;
				item.level = 1;
				item.itemName = 'User has no published tracks';
				item.callbackData = '{"errors": "no tracks"}';
				script.insertItem(item);
				script.donePopulating();
// 			} else if (artistData.errors == 'prompt') {
// 				item = Amarok.StreamItem;
// 				item.level = 1;
// 				item.itemName = 'Search for an artist...';
// 				item.callbackData = '{"errors": "prompt"}';
// 				script.insertItem(item);
// 				script.donePopulating();
			}
			break;
		case 0:
			tracks(callback);
			break;
	}
}

// var dock = new QDockWidget( "Scripted Dock" );
// var list = new QListWidget();
// dock.setWidget( list );
// Amarok.Window.mainWindow.addDockWidget( Qt.LeftDockWidgetArea, dock );
// dock.show();

Amarok.configured.connect( onConfigure );
script = new SoundCloudService();
script.populate.connect( onPopulating );
